var currentDurationCount = 1;     // TBD: Move this to session cookie store?
var currentDurationScale = 'day'; // TBD: Move this to session cookie store?

var zoomScales = ['day', 'week', 'month'];

function initialiseTimelineManipulationHandlers() {
    // Timeline update handlers (see timeline_manipulation)
    $.each(zoomScales, function (index, text) {
        $('#zoom-scale-selector').append(
            $('<option/>')
                .attr('value', text)
                .prop('selected', text == currentDurationScale)
                .text(text)
        );
    });

    attachTimelineManipulationHandlers();
}

function attachTimelineManipulationHandlers() {
    $("#zoom-selector-button").on("change", function () {
        var newScale = $('#zoom-scale-selector').val();
        if (newScale != currentDurationScale) {
            var changed = true;
        }
    });

    $("#zoom-in-button").off("click").on("click", function () {
        timelineZoomHandler('+');
    });

    $("#zoom-out-button").off("click").on("click", function () {
        timelineZoomHandler('-');
    });

    $('#fullcalendar_scheduler .fc-header-toolbar .fc-center h2').datetimepicker({
        timepicker: false,
        format: 'Y-m-d',
        onChangeDateTime: function (dp, $input) {
            updateTimelineDuration($input.val());
        },
    });
}

/**
 * Re-render timeline view with the specified date range
 *
 * @param direction   '+' = increase / '-' = decrease
 */
function timelineZoomHandler(direction) {
    switch (direction) {
        case '+':
            currentDurationCount = currentDurationCount + 1;
            break;
        case '-':
            currentDurationCount = currentDurationCount - 1;
            break;
    }

    updateTimelineDuration();
}

/**
 * Re-render timeline view with the specified date range
 *
 * Examples:
 * updateTimelineDuration()
 * - use the date fields from the form
 *
 * updateTimelineDuration('2019-11-21')
 * - use the supplied start_date + the 'currentDurationCount'
 *
 * updateTimelineDuration('2019-11-21','2019-11-29')
 * - override any form values
 *
 * @param start_date Start date
 * @param end_date   End date
 */
function updateTimelineDuration(start_date, end_date) {
    if (typeof start_date === 'undefined' || start_date === null || start_date.length == 0) {
        start_date = $.trim($("#start_date").val()).substring(0, 10);
    }

    // If we have no 'start_date', from any source, don't do anything
    if (typeof start_date === 'undefined' || start_date === null || start_date.length == 0) {
        return;
    }

    var startDate = new Date(start_date);

    if (typeof end_date === 'undefined' || end_date === null || end_date.length == 0) {
        end_date = dateAdd(startDate, currentDurationCount - 1, $('#zoom-scale-selector').val());
    }

    var endDate = new Date(end_date);

    var conversionDivisor;
    switch ($('#zoom-scale-selector').val()) {
        case 'month':
            // TODO: Fix me
            // 1 month (30 days) = 2592000000 ms
            conversionDivisor = (1000 * 60 * 60 * 24 * 30);
            break;
        case 'week':
            // 1 week = 604800000 ms
            conversionDivisor = (1000 * 60 * 60 * 24 * 7);
            break;
        case 'day':
        default:
            // 1 day =  86400000 ms
            conversionDivisor = (1000 * 60 * 60 * 24);
    }

    currentDurationCount = Math.round((endDate - startDate) / conversionDivisor) + 1;
    if (currentDurationCount < 1) { currentDurationCount = 1; }

    // Adjust timeline
    $('#fullcalendar_scheduler').fullCalendar('changeView', 'timeline', new Date(startDate).yyyymmdd());
    $('#fullcalendar_scheduler').fullCalendar('option', 'visibleRange', {
        start: new Date(startDate).yyyymmdd(),
        end: dateAdd(startDate, currentDurationCount, $('#zoom-scale-selector').val()).yyyymmdd(),
    });
    $('#fullcalendar_scheduler').fullCalendar('render');

    // Update zoom-text
    $('#zoom-text').text(currentDurationCount);
}
